import { system } from "@minecraft/server"
import { randomFloat, clampNumber, getDeltaLocation, angleToDegrees } from "./utils/mathUtils";

export function recalculateDamage(item, damage) {
    if (damage < 0) return;
    if (item !== undefined) {
        const SmiteLevel = item.getComponent('enchantable')?.getEnchantment('smite')?.level;
        const BaneOfArthropodsLevel = item.getComponent('enchantable')?.getEnchantment('bane_of_arthropods')?.level;

        if (SmiteLevel > 0) {
            return damage += 2 * SmiteLevel;
        } else if (BaneOfArthropodsLevel > 0) {
            return damage += 2.5 * BaneOfArthropodsLevel;
        } else {
            return damage;
        }
    } else {
        return damage;
    }
};

export function showDamage(hurtEntity, damageAmount) {
    if (damageAmount > 0) {
        const totalDamage = damageAmount.toFixed(1);
        const x = randomFloat(-0.3, 0.3);
        const y = randomFloat(0.4, 0.9);
        const z = randomFloat(-0.3, 0.3);

        hurtEntity.runCommand(`summon target_dummy:damage_value "${totalDamage}" ~${x} ~${y} ~${z}`);
    }
};

function getHurtAngle(hurtEntity, entity) {
    if (entity !== undefined) {
        const getDifference = getDeltaLocation(hurtEntity.location, entity.location);
        const yaw = hurtEntity.getRotation().y;
        const dx = getDifference.x;
        const dz = getDifference.z;

        return (angleToDegrees(Math.atan2(dz, dx)) - yaw + 360) % 360;
    } else {
        return undefined;
    }
};

export function setAngleAnimation(entity, attacker) {
    const hurtAngle = getHurtAngle(entity, attacker);

    if (hurtAngle !== undefined) {
        entity?.setProperty("target_dummy:hurt_direction", "undefined");
        system.runTimeout(() => {
            if (hurtAngle >= 337.5 || hurtAngle < 22.5) {
                entity?.setProperty("target_dummy:hurt_direction", "right");
            } else if (hurtAngle >= 22.5 && hurtAngle < 67.5) {
                entity?.setProperty("target_dummy:hurt_direction", "back_right");
            } else if (hurtAngle >= 67.5 && hurtAngle < 112.5) {
                entity?.setProperty("target_dummy:hurt_direction", "back");
            } else if (hurtAngle >= 112.5 && hurtAngle < 157.5) {
                entity?.setProperty("target_dummy:hurt_direction", "back_left");
            } else if (hurtAngle >= 157.5 && hurtAngle < 202.5) {
                entity?.setProperty("target_dummy:hurt_direction", "left");
            } else if (hurtAngle >= 202.5 && hurtAngle < 247.5) {
                entity?.setProperty("target_dummy:hurt_direction", "front_left");
            } else if (hurtAngle >= 247.5 && hurtAngle < 292.5) {
                entity?.setProperty("target_dummy:hurt_direction", "front");
            } else {
                entity?.setProperty("target_dummy:hurt_direction", "front_right");
            }
        }, 1);
    } else {
        entity?.triggerEvent("on_hurt_entity");
    }
};